#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::HTML;
use Wanha::HTTP;
use Wanha::DB;
use Wanha::Link;
use Wanha::Common;
use Wanha::Lang;
use Wanha::URLList;
use Wanha::Session;
use Wanha::Nick;


=item main
Main function
=cut
sub main() {
	&Wanha::Session::get_id(); # Update cookie

	&Wanha::HTTP::print_header();
	&Wanha::HTML::print_header();

	my $user = &Wanha::HTTP::get_numeric("user", -1);
	my $seek = &Wanha::HTTP::get_numeric("seek", -1);
	my $n = &Wanha::HTTP::get_numeric("n", -1);

	&Wanha::Session::print_links(
			undef, &Wanha::Link::user($user, $n, $seek));

	if ($seek == -1) {
		&summary($user);
	} else {
		&url_hist($user, $n, $seek);
	}

	&Wanha::HTML::print_footer(&Wanha::Common::process_time());
}


=item summary
Print user summary

0: user
=cut
sub summary($) {
	my ($id) = @_;

	my ($nick) = &Wanha::Nick::get_name($id);
	(defined($nick)) || &Wanha::HTML::bad_param("bad user");

	&Wanha::Lang::printf("<h1><&user-summary&></h1>\n", $nick);

	my @score = &get_score($id, 10);
	($#score != -1) && &print_score(@score);

	my $r = &urls($id, 0, 8);
	if ($r > 0) {
		&urls($id, 1, 4);
	} else {
		&Wanha::Lang::printf("<p><&user-no-urls&></p>\n", $nick);
	}
}


=item url_hist
Print user URL history

0: user
1: n
2: seek
=cut
sub url_hist($$$) {
	my ($id, $n, $seek) = @_;

	my $nick = &Wanha::Nick::get_name($id);
	(defined($nick)) || &Wanha::HTML::bad_param("bad user");

	&Wanha::Lang::printf("<h1><&user-hist-for&></h1>\n", $nick);

	&urls($id, 0, $n, $seek);
}


=item print_score
Print user score

0: array of hash
=cut
sub print_score(@) {
	my @chans = @_;

	my $l_chan = &Wanha::Lang::get("col-channel");
	my $l_fame = &Wanha::Lang::get("col-fame");
	my $l_shame = &Wanha::Lang::get("col-shame");
	my $l_urls = &Wanha::Lang::get("col-urls");
	my $l_ratio = &Wanha::Lang::get("col-points-ratio");
	my $l_wanha = &Wanha::Lang::get("col-wanha");
	my $l_cheat = &Wanha::Lang::get("col-cheat");

	print <<EOF;
<table class="perchan">
	<tr class="ul">
		<th>$l_chan</th>
		<th>$l_urls</th>
		<th>$l_fame</th>
		<th>$l_ratio</th>
		<th>$l_shame</th>
		<th>$l_ratio</th>
		<th>$l_wanha</th>
		<th>$l_cheat</th>
	</tr>
EOF

	my $t_fame = 0;
	my $t_shame = 0;
	my $t_wanha = 0;
	my $t_cheat = 0;
	my $t_urls = 0;
	for (my $i = 0; $i <= $#chans; $i++) {
		my %t = %{$chans[$i]};

		my $id = $t{'id'};
		my $name = $t{'name'};
		my $fame = $t{'fame'};
		my $shame = $t{'shame'};
		my $wanha = $t{'wanha'};
		my $cheat = $t{'cheat'};
		my $count = $t{'url_count'};

		my $a_chan = &Wanha::Link::chan($id);
		my $class = $i % 2 == 0 ? "zebra_c" : "center";

		my $total = $count + $shame + $wanha;
		my $f_ratio = sprintf("%.1f", $fame / $total * 100);
		my $s_ratio = sprintf("%.1f", $shame / $total * 100);

		print <<EOF;
	<tr class="$class">
		<td class="left"><a href="$a_chan">$name</a></td>
		<td>$total</td>
		<td>$fame</td>
		<td>$f_ratio%</td>
		<td>$shame</td>
		<td>$s_ratio%</td>
		<td>$wanha</td>
		<td>$cheat</td>
	</tr>
EOF
		$t_fame += $fame;
		$t_shame += $shame;
		$t_urls += $total;
		$t_wanha += $wanha;
		$t_cheat += $cheat;
		$i++;
	}

	my $l_total = &Wanha::Lang::get("col-total");
	print <<EOF;
	<tr class="sum">
		<td>$l_total</td>
		<td class="center">$t_urls</td>
		<td class="center">$t_fame</td>
		<td class="center"></td>
		<td class="center">$t_shame</td>
		<td class="center"></td>
		<td class="center">$t_wanha</td>
		<td class="center">$t_cheat</td>
	</tr>
</table>
EOF
}


=item urls
Print user URLs

0: user
1: mode, 0 == latest, 1 == top-n
2: n
3: seek
r: 0 if urls, 1 if no urls
=cut
sub urls($$$$) {
	my ($user_id, $mode, $n, $seek) = @_;
	my @urls;
	my @images;
	my $has_more;

	$n = 50 if ($n > 50);

	# get one extra so that we know if there's more
	@urls = &get_urls($user_id, $mode, $n + 1, $seek);
	($#urls == -1 && defined($seek)) && &Wanha::HTML::bad_param("bad seek");
	($#urls == -1) && return 0;
	if ($#urls == $n) {
		$has_more = 1;
		pop @urls;
	} else {
		$has_more = 0;
	}

	if (! defined($seek)) {
		my $id = ($mode == 0) ? "user-n-latest-urls"
			: "user-n-best-urls";
		&Wanha::Lang::printf("<h2><&$id&></h2>\n", $n);
	} else {
		if ($mode == 0) {
			my $first = $urls[0]{'url_id'};
			my $last = $urls[$#urls]{'url_id'};
			&Wanha::Lang::printf("<h2><&user-urls-range&></h2>\n",
					$first, $last);
		} else {
			my $first = $seek + 1;
			my $last = $n + $seek;
			&Wanha::Lang::printf("<h2><&user-pop-urls-range&>" .
					"</h2>\n", $first, $last);
		}
	}

	@images = &Wanha::URLList::print(\@urls,
			"url", "chan", "date", "count");

	($mode == 1) && return 1;

	my $l;
	if (! defined($seek) && $has_more == 1) {
		&Wanha::Common::links_add(&Wanha::Link::user($user_id, 25, $n),
				&Wanha::Lang::get("link-older-urls"));
	} elsif (defined($seek)) {
		&Wanha::Common::links_add(&Wanha::Link::user($user_id),
				&Wanha::Lang::get("link-user-summary"));
		if ($seek > 0) {
			my $tseek = $seek - $n;
			$tseek = 0 if ($tseek < 0);
			&Wanha::Common::links_add(
					&Wanha::Link::user(
						$user_id, $n, $tseek),
					&Wanha::Lang::get("link-newer"));
		}
		if ($has_more == 1) {
			my $tseek = $seek + $n;
			&Wanha::Common::links_add(
					&Wanha::Link::user(
						$user_id, $n, $tseek),
					&Wanha::Lang::get("link-older"));
		}
	}

	&Wanha::Common::print_links();
	&Wanha::Common::link_images(@images);

	return 1;
}


=item get_urls
Get URLs

0: user
1: mode, 0 == latest, 1 == top-n
2: n
3: seek
r: array of hash (url_id, url_url, url_date, url_count, chan_id, chan_name)
=cut
sub get_urls($$$$) {
	my ($user_id, $mode, $n, $seek) = @_;
	my ($query, $result);
	my @ret;

	my $order;
	if ($mode == 0) {
		$order = "urls.time";
	} elsif ($mode == 1) {
		$order = "urls.count";
	} else {
		&Wanha::HTML::bad_param("mode");
	}

	$query = "SELECT urls.url, urls.id, " .
				&Wanha::DB::get_date_fmt("urls.time") . ", " .
				"urls.count, chans.id, chans.chan " .
			"FROM urls, chans " .
			"WHERE urls.nick = &da& " .
				"AND urls.chan = chans.id " .
			"ORDER BY $order DESC " .
			"LIMIT &db&";
	$query .= " OFFSET &dc&" if (defined($seek));

	$query = &Wanha::DB::prepare($query);
	$query = &Wanha::DB::set($query,
			"a", $user_id,
			"b", $n);
	$query = &Wanha::DB::set($query, "c", $seek) if (defined($seek));

	$result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return ();

	while (my ($url_url, $url_id, $url_date, $url_count,
			$chan_id, $chan_name) = &Wanha::DB::get_row($result)) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_date' => $url_date,
			'url_count' => $url_count,
			'chan_id' => $chan_id,
			'chan_name' => $chan_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


=item get_score
Get user score

0: nick id
1: n
r: array of hash (id, url_count, fame, shame, wanha, cheat, name)
=cut
sub get_score($$) {
	my ($nick_id, $n) = @_;
	my $query;
	
	$query = &Wanha::DB::prepare("SELECT chans.id, chans.chan, " .
				"sb.fame, sb.shame, sb.wanha, sb.cheat " .
			"FROM chans, scoreboard AS sb " .
			"WHERE sb.nick = &da& AND sb.chan = chans.id " .
			"ORDER BY sb.fame, sb.shame DESC ".
			"LIMIT &db&");
	$query = &Wanha::DB::set($query,
			"a", $nick_id,
			"b", $n);
	my $result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return;

	my @ret_pre;
	while (my ($id, $name, $fame, $shame, $wanha, $cheat) =
			&Wanha::DB::get_row($result)) {
		my %t = (
			'id' => $id,
			'name' => $name,
			'fame' => $fame,
			'shame' => $shame,
			'cheat' => $cheat,
			'wanha' => $wanha
			);
		push(@ret_pre, \%t);
	}

	my @ret;
	foreach (@ret_pre) {
		my %t = %{$_};
		$query = &Wanha::DB::prepare("SELECT COUNT(id) FROM urls " .
				"WHERE nick = &da& AND chan = &db&");
		$query = &Wanha::DB::set($query,
				"a", $nick_id,
				"b", $t{'id'});
		$result = &Wanha::DB::query($query);
		my ($c) = &Wanha::DB::get_row($result);
		# TODO there must be a prettier way to do this
		$t{'url_count'} = $c;
		push(@ret, \%t);
	}

	return @ret;
}


&main();
exit(0);
